﻿//////////////////////////////////////////////
// PbsMaterial.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Program ----------------------------------

namespace nkGraphics
{
	class Program ;
	class Texture ;
}

/// Includes ---------------------------------

// nkAstraeus
#include "../../../Dll/DllDefines.h"

#include "../Material.h"

#include "PbsMaterialInfo.h"

/// Class ------------------------------------

namespace nkAstraeus
{
	class DLL_ASTRAEUS_EXPORT PbsMaterial final : public Material
	{
		private :

			// Attributes
			// Program used for PBS
			nkGraphics::Program* _program ;

			// Active program info
			PbsMaterialInfo _info ;

		private :

			// Functions
			// Loading related
			bool _prepareProgram () ;
			bool _prepareShader () ;

		public :

			// Constructor, destructor
			PbsMaterial () ;
			~PbsMaterial () ;

			// Getters
			virtual MATERIAL_TYPE getType () const override ;
			nkMaths::Vector getLightDirection () const ;
			nkMaths::Vector getLightColor () const ;
			nkMaths::Vector getAlbedoColor () const ;
			nkMaths::Vector getEnvironmentColor () const ;
			float getMetalness () const ;
			float getRoughness () const ;
			nkGraphics::Texture* getAlbedoTexture () const ;
			nkGraphics::Texture* getMetalnessTexture () const ;
			nkGraphics::Texture* getRoughnessTexture () const ;
			nkGraphics::Texture* getEnvironmentTexture () const ;
			nkGraphics::Texture* getIrradianceTexture () const ;
			bool getEnvironmentTextureIsCubeByDefault () const ;
			bool getIrradianceTextureIsCubeByDefault () const ;

			// Setters
			void setLightDirection (const nkMaths::Vector& value) ;
			void setLightColor (const nkMaths::Vector& value) ;
			void setAlbedoColor (const nkMaths::Vector& value) ;
			void setEnvironmentColor (const nkMaths::Vector& value) ;
			void setMetalness (float value) ;
			void setRoughness (float value) ;
			void setAlbedoTexture (nkGraphics::Texture* value) ;
			void setMetalnessTexture (nkGraphics::Texture* value) ;
			void setRoughnessTexture (nkGraphics::Texture* value) ;
			void setEnvironmentTexture (nkGraphics::Texture* value) ;
			void setIrradianceTexture (nkGraphics::Texture* value) ;
			void setEnvironmentTextureIsCubeByDefault (bool value) ;
			void setIrradianceTextureIsCubeByDefault (bool value) ;

			// Load
			virtual bool load () override ;
			virtual void unload () override ;
	} ;
}