﻿//////////////////////////////////////////////
// ArrayData.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkAstraeus
{
	template <typename T> struct ArrayView ;
}

/// Includes ---------------------------------

// nkAstraeus
#include "../../../Dll/DllDefines.h"

/// Class ------------------------------------

namespace nkAstraeus
{
	template <typename T>
	struct ArrayData
	{
		// Attributes
		// Le pointeur et la taille tableau
		T* _ptr = nullptr ;
		size_t _size = 0 ;

		// Functions
		// Destructor
		virtual ~ArrayData () ;

		// Sub view pour accès à des sous parties
		ArrayView<T>* subView (size_t offset, size_t size) ;
	} ;
}

/// Utilities --------------------------------

namespace nkAstraeus
{
	template <typename T>
	struct ArrayView : public ArrayData<T>
	{
		// Override destructor pour refléter l'ownership inexistante
		~ArrayView ()
		{
			this->_ptr = nullptr ;
		}
	} ;
}

/// Définitions ------------------------------

namespace nkAstraeus
{
	using ByteArrayData = ArrayData<char> ;
	using ByteArrayView = ArrayView<char> ;

	using UbyteArrayData = ArrayData<unsigned char> ;
	using UbyteArrayView = ArrayView<unsigned char> ;
}

/// Functions --------------------------------
/// Destructor -------------------------------

template <typename T>
nkAstraeus::ArrayData<T>::~ArrayData ()
{
	delete[] _ptr ;
	_ptr = nullptr ;
}

template <typename T>
nkAstraeus::ArrayView<T>* nkAstraeus::ArrayData<T>::subView (size_t offset, size_t size)
{
	ArrayView<T>* subView = new ArrayView<T> () ;
	subView->_ptr = _ptr + offset ;
	subView->_size = size ;

	return subView ;
}