﻿//////////////////////////////////////////////
// LuaTask.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkScripts
{
	class Environment ;
	class Script ;
}

/// Includes ---------------------------------

// nkAstraeus
#include "../../Arrays/ArrayData.h"

// nkScripts
#include <NilkinsScripts/Environments/Functions/FunctionDefines.h>

#include <NilkinsScripts/Environments/ScriptObjects/ScriptObjectReference.h>

// nkTasks
#include <NilkinsTasks/Tasks/Task.h>

// Standards
#include <vector>

/// Internals --------------------------------

namespace nkAstraeus::lua::nkTasksWrap
{
	struct UserDataHolder
	{
		std::string _typeName ;
		void* _userData = nullptr ;
		bool _owner = false ;

		UserDataHolder (void* userData, const char* typeName, bool owner)
		:	_typeName (typeName),
			_userData (userData),
			_owner (owner)
		{
			// Nothing to do
		}
	} ;

	struct BinaryDataHolder
	{
		UbyteArrayData* _data = nullptr ;
		nkScripts::ScriptObjectReference _ref ;
	} ;
}

/// Class ------------------------------------

namespace nkAstraeus::lua::nkTasksWrap
{
	class LuaTask final : public nkTasks::Task
	{
		private :

			// Environnements création / work
			nkScripts::Environment* _originEnv ;
			nkScripts::Environment* _workEnv ;

			// Les fonctions nécessaires sérialisées
			std::vector<unsigned char> _workFuncSerialized ;
			// Userdata à réinjecter
			std::vector<UserDataHolder> _inputUserDataSerialized ;
			std::vector<UserDataHolder> _outputUserDataSerialized ;

			// Le script d'init lors du lancement
			nkScripts::Script* _initScript ;

			// Stack d'output toute prête
			nkScripts::DataStack _outputStack ;

			// Input output binaires
			BinaryDataHolder _inputBinaryData ;
			UbyteArrayData* _outputBinaryData ;

		public :

			// Constructeur, destructeur
			LuaTask (nkScripts::Environment* originEnv) ;
			~LuaTask () ;

			// Getters
			UbyteArrayData* getInputBinaryData () const ;
			UbyteArrayData* getOutputBinaryData () const ;

			// Setters
			void setWorkEnv (nkScripts::Environment* workEnv) ;
			void setWorkFuncReference (const nkScripts::ScriptObjectReference& reference) ;
			void setInitScript (nkScripts::Script* script) ;
			void setInputBinaryData (UbyteArrayData* data, const nkScripts::ScriptObjectReference& ref) ;
			void setOutputBinaryData (UbyteArrayData* data) ;
			
			// Manipulation user data
			void addInputUserData (void* ptr, const char* typeName, bool owner) ;
			void addOutputUserData (void* ptr, const char* typeName, bool owner) ;
			const UserDataHolder* getInputUserData (unsigned int index) const ;
			const UserDataHolder* getOutputUserData (unsigned int index) const ;

			// Réimplémentations
			virtual nkTasks::TASK_RESULT onWork () override ;
			virtual void onWorkDone () override ;
			virtual void onSuccess () override ;
			virtual void onFail () override ;
			virtual void onAbort () override ;
	} ;
}