﻿//////////////////////////////////////////////
// ShaderPassMemorySlot.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Camera ;
	class ShaderPassCustomSlot ;
	class Texture ;
}

namespace nkMaths
{
	class Matrix ;
	class Vector ;
}

/// Includes ---------------------------------

// nkGraphics
#include "ShaderMemorySlot.h"

#include "../../Dll/DllDefines.h"

/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT ShaderPassMemorySlot : public ShaderMemorySlot
	{
		public :

			enum class DATA_TYPE
			{
				// Global et inconnu
				UNKNOWN = 0,
				VECTOR,
				MATRIX,
				// Plutôt pour le post process
				CAM_CORNERS_WORLD,
				CAM_CORNERS_VIEW,
				TARGET_SIZE,
				PROJECTION_PARAMS,
				TEXTURE_SIZE,
				// Plutôt pour les rendus de scène
				VIEW_MATRIX,
				VIEW_MATRIX_INV,
				PROJ_MATRIX,
				PROJ_MATRIX_INV,
				CAM_POSITION,
				CAM_DIRECTION_WORLD,
				// Utile
				TIME,
				// Custom
				CUSTOM
			} ;

		protected :
		
			// Attributs
			// Les données et leur taille
			void* _data ;
			unsigned int _dataSize ;

			// Le type de données
			DATA_TYPE _type ;

		protected :

			// Functions
			// Data erasure
			void _deleteData () ;
		
		public :
		
			// Constructeur, destructeur
			ShaderPassMemorySlot () ;
			virtual ~ShaderPassMemorySlot () ;

			// Getters
			void* getData () const ;
			virtual unsigned int getDataSize () const ;
			virtual const char* getDataTypeAsString () const override ;
			DATA_TYPE getType () const ;
			nkMaths::Vector* getDataAsVector () const ;
			nkMaths::Matrix* getDataAsMatrix () const ;

			// Setters
			void setFromVector (const nkMaths::Vector& data) ;
			void setFromMatrix (const nkMaths::Matrix& data) ;
			void setAsCamCornersWorld (Camera* forcedCamera = nullptr) ;
			void setAsCamCornersView (Camera* forcedCamera = nullptr) ;
			void setAsTargetSize () ;
			void setAsProjectionParameters (Camera* forcedCamera = nullptr) ;
			void setAsViewMatrix (Camera* forcedCamera = nullptr) ;
			void setAsViewMatrixInv (Camera* forcedCamera = nullptr) ;
			void setAsProjectionMatrix (Camera* forcedCamera = nullptr) ;
			void setAsProjectionMatrixInv (Camera* forcedCamera = nullptr) ;
			void setAsCameraPosition (Camera* forcedCamera = nullptr) ;
			void setAsCameraDirectionWorld (Camera* forcedCamera = nullptr) ;
			void setAsTextureSize (Texture* tex) ;
			void setAsTime (unsigned int modulo) ;
			void setAsCustom (ShaderPassCustomSlot* slot) ;

			// Import / export
			virtual void exportClassToTree (nkExport::Node* rootNode) override ;
			virtual void importClassFromTree (nkExport::Node* rootNode) override ;
	} ;
}